//
//  ButtonCell.m
//  iShaker
//
//  Created by mike on 05/01/2009.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "MenuButtonCell.h"
#import "MenuConstants.h"
#import "MenuViewController.h"

#define LABEL_TAG 1

@implementation MenuButtonCell
@synthesize userDefaultName = _userDefaultName;

- (id)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier {
  self = [super initWithStyle:style reuseIdentifier:reuseIdentifier];
  if (self) {
    self.textLabel.lineBreakMode = NSLineBreakByTruncatingTail;
    self.textLabel.backgroundColor = [UIColor clearColor];
    
    // custom for a regular button cell
    self.textLabel.textAlignment = NSTextAlignmentCenter;
    self.accessoryType = UITableViewCellAccessoryNone;
  }
  return self;
}

- (void)dealloc {
  self.userDefaultName = nil;
  [super dealloc];
}

-(void) loadWithDictionary:(NSDictionary*)dictionary menu:(MenuViewController*)menu {
  [super loadWithDictionary:dictionary menu:menu];
  [menu customizeCellTitle:self];
  
  NSString* title = [MenuViewController stringForMenu:dictionary key:MenuItemTitle];
  [self setTitle:title];
  
  SEL selector = NSSelectorFromString([dictionary valueForKey:MenuItemSelector]);
  if (selector) {
    [self setTarget:menu.delegate selector:selector];
  }
  
  selector = NSSelectorFromString([dictionary valueForKey:MenuItemChangeSelector]);
  if (selector) {
    [self setTarget:menu.delegate changeSelector:selector];
  }
  
  // userDefaultName
  NSString* userDefaultName = [dictionary objectForKey:MenuItemUserDefaultName];
  if (userDefaultName) {
    self.userDefaultName = userDefaultName;
  }
}

- (void)setTitle:(NSString*)newTitle {
  self.textLabel.text = newTitle;
  [self setNeedsDisplay];
}

- (void)setTarget:(id)target selector:(SEL)selector {
  if ([target respondsToSelector:selector]) {
    actionTarget = target;
    actionSelector = selector;
  } else {
    DLog(@"target of button '%@' does not respond to selector '%@'", self.textLabel.text, NSStringFromSelector(selector));
  }
}

-(void) setTarget:(id)target changeSelector:(SEL)selector {
  if ([target respondsToSelector:selector]) {
    changeTarget = target;
    changeSelector = selector;
  } else {
    DLog(@"change target of button '%@' does not respond to selector '%@'", self.textLabel.text, NSStringFromSelector(selector));
  }
}

- (void)setSelected:(BOOL)selected animated:(BOOL)animated {
  [super setSelected:NO animated:animated];
  
  if (selected) {
    [self execute];
  }
}

-(void) setValue:(id)value {}

-(void) setUserDefault:(id)value {
  [[NSUserDefaults standardUserDefaults] setObject:value forKey:_userDefaultName];
  [[NSUserDefaults standardUserDefaults] synchronize];
}

-(void) setUserDefaultName:(id)value {
  if (_userDefaultName != value) {
    [_userDefaultName release];
    _userDefaultName = [value retain];
  }
  
  if (!_userDefaultName) return;

  id userDefaultValue = [[NSUserDefaults standardUserDefaults] objectForKey:_userDefaultName];
  if (userDefaultValue) {
    [self setValue:userDefaultValue];
  }
}

- (void)execute {
  [actionTarget performSelector:actionSelector];
}



@end
