//
//  ComboListViewController.m
//  iShaker
//
//  Created by mike on 02/02/2009.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "MenuComboListViewController.h"
#import "MenuConstants.h"
#import "MenuComboCell.h"

#define MenuItemKindComboButton @"comboButton"

@implementation MenuViewController(Combo)

+(NSString*) itemValue:(id)item {
  NSString* value = nil;
  if ([item isKindOfClass:[NSString class]]) {
    value = item;
  } else if ([item isKindOfClass:[NSDictionary class]]) {
    value = [item valueForKey:@"value"];
  } else {
    DLog(@"MenuViewController itemValue: unknown combo item type %@", NSStringFromClass([item class]));
    value = @"";
  }
  return value;
}

+(NSString*) itemTitle:(id)item {
  NSString* title = nil;
  if ([item isKindOfClass:[NSString class]]) {
    title = item;
  } else if ([item isKindOfClass:[NSDictionary class]]) {
    title = [item valueForKey:@"title"];
  } else {
    DLog(@"MenuViewController itemTitle: unknown combo item type %@", NSStringFromClass([item class]));
    title = @"";
  }
  return title;
}

@end

@implementation MenuComboListViewController

@synthesize comboCell = _comboCell;
@synthesize items;
@synthesize userDefaultName = _userDefaultName;
@synthesize selectedPath = _selectedPath;

-(void)dealloc {
  DLog(@"dealloc %@", [self class]);
  self.comboCell = nil;
  self.items = nil;
  self.userDefaultName = nil;
  self.selectedPath = nil;
  [super dealloc];
}

- (void)viewDidLoad {
  [super viewDidLoad];
}

- (void)setTarget:(id)target selector:(SEL)selector {
  if ([target respondsToSelector:selector]) {
    actionTarget = target;
    actionSelector = selector;
  } else {
    DLog(@"target of combo '%@' does not respond to selector '%@'", self.comboCell.textLabel.text, NSStringFromSelector(selector));
  }
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
  return 1;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)sectionIndex {  
  return nil;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)sectionIndex {
  return [self.items count];
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
  MenuCell* cell = (MenuCell*)[tableView dequeueReusableCellWithIdentifier:MenuItemKindComboButton];
	if (cell == nil) {
      cell = [[[MenuCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:MenuItemKindComboButton] autorelease];
	}
  
  cell.textLabel.text = [MenuViewController itemTitle:[self.items objectAtIndex:indexPath.row]];
  cell.selectionStyle = UITableViewCellSelectionStyleNone;

  cell.textLabel.textAlignment = NSTextAlignmentLeft;
  cell.textLabel.lineBreakMode = NSLineBreakByTruncatingTail;
  cell.textLabel.backgroundColor = [UIColor clearColor];

  [self customizeCellTitle:cell];
  [self tableView:tableView customizeCellBackground:cell indexPath:indexPath];
  
  if ([self isSelectedAtIndexPath:indexPath]) {
    UIImageView* tickView = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"MenuComboCheck.png"]];
    cell.accessoryView = tickView;
    [tickView release];
    self.selectedPath = indexPath;
  }
  
  return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)newIndexPath {
  UITableViewCell* cell;
  
  // deselect old selected cell (if any)
  if (self.selectedPath) {
    cell = [tableView cellForRowAtIndexPath:self.selectedPath];
    cell.accessoryView = nil;
  }

  // select the new cell
  self.selectedPath = newIndexPath;
  cell = [tableView cellForRowAtIndexPath:newIndexPath];
  UIImageView* tickView = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"MenuComboCheck.png"]];
  cell.accessoryView = tickView;
  [tickView release];
  
  // update the userDefault
  if (self.userDefaultName) {
    id value = [self.items objectAtIndex:newIndexPath.row];
    [[NSUserDefaults standardUserDefaults] setObject:value forKey:self.userDefaultName];
    [[NSUserDefaults standardUserDefaults] synchronize];
    
    // execute the target selector (if any)
    [actionTarget performSelector:actionSelector withObject:self.userDefaultName withObject:value];  
  }
}

- (void)viewDidDisappear:(BOOL)animated {
  if (self.comboCell) {
    if (self.selectedPath.length) {
      // sunchronize defaults only after returingin form the combo, since doing it for every
      // selection change is not necessary
      [[NSUserDefaults standardUserDefaults] synchronize];

      // update the combo cell
      id item = [self.items objectAtIndex:self.selectedPath.row];
      [self.comboCell setValue:item];
      
      // reload the data to make sure that the first time we go back the selected item is shown
      [self.comboCell.menuViewController.tableView reloadData];
    }
  }
}

-(BOOL) isSelectedAtIndexPath:(NSIndexPath*)indexPath {
  if (self.userDefaultName) {
    NSUserDefaults* userDefaults = [NSUserDefaults standardUserDefaults];
    NSString* userDefaultValue = [MenuViewController itemValue:[userDefaults objectForKey:self.userDefaultName]];
    NSString* selected = [MenuViewController itemValue:[self.items objectAtIndex:indexPath.row]];
    return [selected isEqualToString:userDefaultValue];
  } else {
    DLog(@"no userDefaultName for %@", [self.items objectAtIndex:indexPath.row]);
    return NO;
  }
}




@end
