//
//  SwitchCell.m
//  iShaker
//
//  Created by mike on 07/01/2009.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "MenuSliderCell.h"
#import "MenuConstants.h"
#import "MenuViewController.h"

@implementation MenuSliderCell

@synthesize userDefaultName = _userDefaultName;

- (id)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier {
	self = [super initWithStyle:style reuseIdentifier:reuseIdentifier];
  if (self) {
    _slider = [[UISlider alloc] initWithFrame:
                  CGRectMake(kMenuSliderX, kMenuSliderY, kMenuSliderWidth, kMenuSliderHeight)];
    _slider.minimumValueImage = [UIImage imageNamed:@"volume-low.png"];
    _slider.maximumValueImage = [UIImage imageNamed:@"volume-high.png"];
    _slider.continuous = NO;
    [_slider addTarget:self
                  action:@selector(changed:)
        forControlEvents:UIControlEventValueChanged];
    [self.contentView addSubview:_slider];
    [_slider release];    
            
    self.selectionStyle = UITableViewCellSelectionStyleNone;
	}
	return self;
}

- (void)dealloc {
  self.userDefaultName = nil;
	[super dealloc];
}

-(void) loadWithDictionary:(NSDictionary*)dictionary menu:(MenuViewController*)menu {
  [super loadWithDictionary:dictionary menu:menu];

  SEL selector = NSSelectorFromString([dictionary valueForKey:MenuItemSelector]);
  if (selector) {
    [self setTarget:menu.delegate selector:selector];
  }
  
  NSString* userDefaultName = [dictionary objectForKey:MenuItemUserDefaultName];
  if (userDefaultName) {
    self.userDefaultName = userDefaultName;
  }
 
}

- (void)setTitle:(NSString*)newTitle {
  self.textLabel.text = newTitle;
  [self setNeedsDisplay];
}

- (void)changed:(id)sender {
  // update the userDefault
  if (self.userDefaultName) {
    [[NSUserDefaults standardUserDefaults] setFloat:_slider.value forKey:self.userDefaultName];
  }
  DLog(@"changed: %f", _slider.value);
  // execute the target selector
  [actionTarget performSelector:actionSelector withObject:self.userDefaultName 
  withObject:[NSNumber numberWithFloat:_slider.value]];
}

- (void)setTarget:(id)target selector:(SEL)selector {
  if ([target respondsToSelector:selector]) {
    actionTarget = target;
    actionSelector = selector;
  } else {
    DLog(@"target of button '%@' does not respond to selector '%@'", 
    self.textLabel.text, NSStringFromSelector(selector));
  }
}

-(void) setUserDefaultName:(NSString*)value {
  if (_userDefaultName != value) {
    [_userDefaultName release];
    _userDefaultName = [value retain];
  }
  
  if (!_userDefaultName) return;
  
  NSUserDefaults* userDefaults = [NSUserDefaults standardUserDefaults];
  _slider.value = [userDefaults floatForKey:self.userDefaultName];
}


@end
