//
//  SwitchCell.m
//  iShaker
//
//  Created by mike on 07/01/2009.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "MenuSwitchCell.h"
#import "MenuConstants.h"
#import "MenuViewController.h"

@implementation MenuSwitchCell

@synthesize userDefaultName = _userDefaultName;

- (id)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier {
	self = [super initWithStyle:style reuseIdentifier:reuseIdentifier];
  if (self) {
    self.textLabel.textAlignment = NSTextAlignmentLeft;
    self.textLabel.lineBreakMode = NSLineBreakByTruncatingTail;
    self.textLabel.backgroundColor = [UIColor clearColor];
        
    switchObj = [[UISwitch alloc] initWithFrame:CGRectMake(1.0, 1.0, 20.0, 20.0)];
    switchObj.on = NO;
    self.accessoryView = switchObj;
    
    // setup the default handler
    [switchObj addTarget:self
                  action:@selector(selected:)
        forControlEvents:UIControlEventValueChanged];
            
    self.selectionStyle = UITableViewCellSelectionStyleNone;
	}
	return self;
}

- (void)dealloc {
  [switchObj release];
  self.userDefaultName = nil;
	[super dealloc];
}

-(void) loadWithDictionary:(NSDictionary*)dictionary menu:(MenuViewController*)menu {
  [super loadWithDictionary:dictionary menu:menu];
  [menu customizeCellTitle:self];

  NSString* title = [MenuViewController stringForMenu:dictionary key:MenuItemTitle];
  [self setTitle:title];
  
  SEL selector = NSSelectorFromString([dictionary valueForKey:MenuItemSelector]);
  if (selector) {
    [self setTarget:menu.delegate selector:selector];
  }
  
  NSString* userDefaultName = [dictionary objectForKey:MenuItemUserDefaultName];
  if (userDefaultName) {
    self.userDefaultName = userDefaultName;
  }  
}

- (void)setTitle:(NSString*)newTitle {
  self.textLabel.text = newTitle;
  [self setNeedsDisplay];
}

- (void)selected:(id)sender {
  // update the userDefault
  if (self.userDefaultName) {
    [[NSUserDefaults standardUserDefaults] setBool:switchObj.on forKey:self.userDefaultName];
  }
  
  // execute the target selector
  [actionTarget performSelector:actionSelector withObject:self.userDefaultName withObject:[NSNumber numberWithBool:switchObj.on]];
}

- (void)setTarget:(id)target selector:(SEL)selector {
  if ([target respondsToSelector:selector]) {
    actionTarget = target;
    actionSelector = selector;
  } else {
    DLog(@"target of button '%@' does not respond to selector '%@'", self.textLabel.text, NSStringFromSelector(selector));
  }
}


-(void) setUserDefaultName:(NSString*)value {
  if (_userDefaultName != value) {
    [_userDefaultName release];
    _userDefaultName = [value retain];
  }

  if (!_userDefaultName) return;
  
  NSUserDefaults* userDefaults = [NSUserDefaults standardUserDefaults];
  switchObj.on = [userDefaults boolForKey:_userDefaultName];
}

@end
