//
//  MainMenuViewController.m
//  iShaker
//
//  Created by mike on 05/01/2009.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "MenuViewController.h"
#import "MenuConstants.h"
#import "MenuButtonCell.h"
#import "MenuSwitchCell.h"
#import "MenuDrillDownCell.h"
#import "MenuComboCell.h"
#import "MenuSliderCell.h"
#import "MenuVolumeCell.h"
#import "MenuEntryCell.h"

@interface MenuViewController(Private)
@end

static UINavigationController* _navigationController = nil;
static UIImage* _backgroundImage = nil;
static UIImage* _top = nil;
static UIImage* _topPressed = nil;
static UIImage* _middle = nil;
static UIImage* _middlePressed = nil;
static UIImage* _bottom = nil;
static UIImage* _bottomPressed = nil;
static UIImage* _single = nil;
static UIImage* _singlePressed = nil;
static UIFont* _cellTitleFont = nil;
static UIColor* _cellTitleColor = nil;
static UIColor* _cellTitleShadowColor = nil;
static CGSize _cellTitleShadowOffset = {0, 0};
static NSMutableDictionary* _valueStore = nil;

@implementation MenuViewController

@synthesize menu = _menu;
@synthesize delegate = _delegate;

+(void) initialize {
  if (!_valueStore) {
    _valueStore = [[NSMutableDictionary alloc] init];
  }
}

+ (void)setNavigationController:(UINavigationController*)controller {
  _navigationController = controller;
}

+ (UINavigationController*)navigationController {
  return _navigationController;
}

- (id)init {
  return [self initWithStyle:UITableViewStyleGrouped];
}

- (id) initWithPlist:(NSString*)plistName {
  self = [self initWithStyle:UITableViewStyleGrouped];
  if (self) {
    NSString *path=[[NSBundle mainBundle] pathForResource:plistName ofType:@"plist"];
    if (path) {
      self.menu = [[NSDictionary alloc] initWithContentsOfFile:path];
    } else {
      DLog(@"%@ could not find %@.plist", [self class], plistName);
    }
  }
  return self;
}

- (id)initWithStyle:(UITableViewStyle)style {
  self = [super initWithStyle:style];
	if (self) {
    self.delegate = self;
	}
	return self;
}

- (void)dealloc {
  self.menu = nil;
  self.delegate = nil;
	[super dealloc];
}

-(void) viewDidLoad {
  [super viewDidLoad];
  
  // set the background color, taking preference for plist defined ...
  NSString* backgroundColor = [self.menu valueForKey:MenuItemBackgroundColor];
  if (backgroundColor) {
    self.view.backgroundColor = [MenuViewController colorFromHexString:backgroundColor];
    self.tableView.backgroundView = nil;
  
  // ... otherwise try to set the staticly defined background
  } else if (_backgroundImage) {
    self.view.backgroundColor = [UIColor colorWithPatternImage:_backgroundImage];
    self.tableView.backgroundView = nil;
  }
  
  // disable bouncing
  self.tableView.bounces = [[self.menu valueForKey:MenuItemBounce] boolValue];
}

#pragma mark Value store

+(void) setValue:(id)value forKey:(NSString*)key {
  [_valueStore setValue:value forKey:key];
}

+(id) valueForKey:(NSString*)key {
  return [_valueStore valueForKey:key];
}

#pragma mark Customize visuals

// taken from http://stackoverflow.com/a/12397366
+ (UIColor *)colorFromHexString:(NSString *)hexString {
    unsigned rgbValue = 0;
    NSScanner *scanner = [NSScanner scannerWithString:hexString];
    [scanner setScanLocation:1]; // bypass '#' character
    [scanner scanHexInt:&rgbValue];
    return [UIColor colorWithRed:((rgbValue & 0xFF0000) >> 16)/255.0 green:((rgbValue & 0xFF00) >> 8)/255.0 blue:(rgbValue & 0xFF)/255.0 alpha:1.0];
}

+(void) setBackgroundImage:(UIImage*)image {
  if (_backgroundImage != image) [_backgroundImage release];
  _backgroundImage = [image retain];
}

+(void) setCellImageTop:(UIImage*)top
             topPressed:(UIImage*)topPressed
                 middle:(UIImage*)middle
          middlePressed:(UIImage*)middlePressed
                 bottom:(UIImage*)bottom
          bottomPressed:(UIImage*)bottomPressed
                 single:(UIImage*)single
          singlePressed:(UIImage*)singlePressed
{
  if (_top != top) [_top release];
  _top = [top retain];
  if (_topPressed != topPressed) [_topPressed release];
  _topPressed = [topPressed retain];
  
  if (_middle != middle) [_middle release];
  _middle = [middle retain];
  if (_middlePressed != middlePressed) [_middlePressed release];
  _middlePressed = [middlePressed retain];
  
  if (_bottom != bottom) [_bottom release];
  _bottom = [bottom retain];
  if (_bottomPressed != bottomPressed) [_bottomPressed release];
  _bottomPressed = [bottomPressed retain];
  
  if (_single != single) [_single release];
  _single = [single retain];
  if (_singlePressed != singlePressed) [_singlePressed release];
  _singlePressed = [singlePressed retain];
}

+(void) setCellTitleFont:(UIFont*)font {
  _cellTitleFont = font;
}

+(void) setCellTitleColor:(UIColor*)color {
  _cellTitleColor = color;
}
+(void) setCellTitleShadowColor:(UIColor*)color {
  _cellTitleShadowColor = color;
}
+(void) setCellTitleShadowOffset:(CGSize)offset {
  _cellTitleShadowOffset = offset;
}

-(void) tableView:(UITableView *)tableView customizeCellBackground:(UITableViewCell*)cell indexPath:(NSIndexPath*)indexPath {
  // check if custom backrounds were specified
  if (!_top) return;
  
  // craete the cell background dependant on its position in the section
  UIImage *rowBackground;
  UIImage *selectionBackground;
  NSInteger sectionRows = [tableView numberOfRowsInSection:[indexPath section]];
  NSInteger row = [indexPath row];
  if (row == 0 && row == sectionRows - 1) {
    rowBackground = _single;
    selectionBackground = _singlePressed;
  } else if (row == 0) {
    rowBackground = _top;
    selectionBackground = _topPressed;
  } else if (row == sectionRows - 1) {
    rowBackground = _bottom;
    selectionBackground = _bottomPressed;
  } else {
    rowBackground = _middle;
    selectionBackground = _middlePressed;
  }
  
  UIImageView *backView = [[UIImageView alloc] initWithImage:rowBackground];
  cell.backgroundView = backView;
  [backView release];
  
  backView =[[UIImageView alloc] initWithImage:selectionBackground];
  cell.selectedBackgroundView = backView;
  [backView release];
}

-(void) customizeCellTitle:(MenuCell*)cell {
  if (_cellTitleFont) cell.textLabel.font = _cellTitleFont;
  if (_cellTitleColor) cell.textLabel.textColor = _cellTitleColor;
  if (_cellTitleShadowColor) cell.textLabel.shadowColor = _cellTitleShadowColor;
  cell.textLabel.shadowOffset = _cellTitleShadowOffset;
}

#pragma mark Utitilies

-(void) setMenu:(NSDictionary*)newMenu {
  if (_menu != newMenu) {
    [_menu release];
    _menu = [newMenu retain];
  }
  
  if (!_menu) { self.title = [_menu valueForKey:MenuItemTitle]; return; }
  
  // check if the menu items are vaild
  for (NSString* required in [NSArray arrayWithObjects:MenuItemKind, MenuItemTitle, nil]) {
    if (![_menu valueForKey:required]) {
      DLog(@"required menu item missing '%@'", required);
    }
  }
  self.title = [_menu valueForKey:MenuItemTitle];
}

+(NSString*) stringForMenu:(NSDictionary*)menu key:(NSString*)key {
  NSString* title = [menu valueForKey:key];
  
  if (title) {
    NSString* value = [[NSUserDefaults standardUserDefaults] stringForKey:title];
    if (value) { title = value; }
  }
  
  return title;
}

#pragma mark Sections

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
  NSArray* sections = [self.menu objectForKey:MenuItemSections];
  if (!sections) {
    DLog(@"no sections for menu '%@'", [self.menu objectForKey:MenuItemTitle]);
    return 1;
  }
  
	return [sections count];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)sectionIndex {
  NSArray* sections = [self.menu objectForKey:MenuItemSections];
  NSDictionary* section = [sections objectAtIndex:sectionIndex];
  NSArray* items = [section objectForKey:MenuItemItems];
  return [items count];
}

#pragma mark Header & Footer

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
  NSArray* sections = [self.menu objectForKey:MenuItemSections];
  return [MenuViewController stringForMenu:[sections objectAtIndex:section] key:MenuItemTitle];
}

- (NSString *)tableView:(UITableView *)tableView titleForFooterInSection:(NSInteger)section {
  NSArray* sections = [self.menu objectForKey:MenuItemSections];
  return [MenuViewController stringForMenu:[sections objectAtIndex:section] key:MenuItemFooter];
}

#pragma mark cellForRowAtIndexPath

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
  
  // get the menu item indexed by the path
  NSArray* sections = [self.menu objectForKey:MenuItemSections];
  NSDictionary* section = [sections objectAtIndex:indexPath.section];
  NSArray* items = [section objectForKey:MenuItemItems];
  NSDictionary* item = [items objectAtIndex:indexPath.row];
  NSString *kind = [item valueForKey:MenuItemKind];
  
  // get the appropriate cell for the item, either new or reused
  MenuCell* cell = (MenuCell*)[tableView dequeueReusableCellWithIdentifier:kind];
	if (cell == nil) {
    if ([kind isEqualToString:MenuItemKindButton]) {
      cell = [[[MenuButtonCell alloc] initWithReuseIdentifier:MenuItemKindButton] autorelease];
      
    } else if ([kind isEqualToString:MenuItemKindSwitch]) {
      cell = [[[MenuSwitchCell alloc] initWithReuseIdentifier:MenuItemKindSwitch] autorelease];
      
    } else if ([kind isEqualToString:MenuItemKindMenu]) {
      cell = [[[MenuDrillDownCell alloc] initWithReuseIdentifier:MenuItemKindMenu] autorelease];
      
    } else if ([kind isEqualToString:MenuItemKindCombo]) {
      cell = [[[MenuComboCell alloc] initWithReuseIdentifier:MenuItemKindCombo] autorelease];
      
    } else if ([kind isEqualToString:MenuItemKindSlider]) {
      cell = [[[MenuSliderCell alloc] initWithReuseIdentifier:MenuItemKindSlider] autorelease];
      
#if MENU_VOLUME_CELL_EN > 0      
    } else if ([kind isEqualToString:MenuItemKindVolume]) {
      cell = [[[MenuVolumeCell alloc] initWithReuseIdentifier:MenuItemKindVolume] autorelease];
#endif
      
    } else if ([kind isEqualToString:MenuItemKindEntry]) {
      cell = [[[MenuEntryCell alloc] initWithReuseIdentifier:MenuItemKindEntry] autorelease];
      
    } else {
      DLog(@"unknown cell type '%@'", kind);
    }
	}
  
  [self tableView:tableView customizeCellBackground:cell indexPath:indexPath];
  
  // now that we have a cell (either new or reused), we can configure it
  [cell loadWithDictionary:item menu:self];
  
  return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)newIndexPath {
  //[tableView deselectRowAtIndexPath:[tableView indexPathForSelectedRow] animated:NO];
}

- (void)didReceiveMemoryWarning {
	[super didReceiveMemoryWarning];
}


@end

