//
//  ViewController.m
//  SmartLight
//
//  Created by Mike on 13/9/13.
//  Copyright (c) 2013 Mike. All rights reserved.
//

#import "ViewController.h"
#import "LightViewController.h"
#import "DiscreteLightViewController.h"
#import "BrightnessLightViewController.h"

#define kViewWidth 320
#define kUserUri @"mike"

@interface ViewController ()

@end

@implementation ViewController

- (void)viewDidLoad {
  [super viewDidLoad];
  self.title = @"Light Control";

  // setup the settings button in the navigation bar
  UIBarButtonItem *rightButton = [[UIBarButtonItem alloc] initWithTitle:@"Settings" 
                                                                  style:UIBarButtonItemStylePlain 
                                                                 target:self
                                                                 action:@selector(onSettingsPress:)];          
  self.navigationItem.rightBarButtonItem = rightButton;
  [rightButton release];
  
  // setup the array for storing the light controllers
  self.lightControllers = [[NSMutableArray alloc] init];

  // join the smart space
  //[self onJoinPress];
}

- (void)didReceiveMemoryWarning {
  [super didReceiveMemoryWarning];
  // Dispose of any resources that can be recreated.
}

-(void) viewRefresh {
  // remove any views loaded for previous session, if any
  for (LightViewController* controller in self.lightControllers) {
    [controller.view removeFromSuperview];
  }
  [self.lightControllers removeAllObjects];
  
  if (self.policyController) {
    [self.policyController.view removeFromSuperview];
    self.policyController = nil;
  }
  
  [MenuViewController setValue:nil forKey:@"policies"];

  // populate the policies list with available policies
  KPLightPolicy policies[kKPMaxLightPolicies];
  int numPolicies = KPGetLightPolicies(policies, kKPMaxLightPolicies);
  
  // get the policies and their uris into the menu
  NSMutableArray* policiesArray = [[NSMutableArray alloc] init];
  for (int i = 0; i < numPolicies; i++) {
    NSString* user = [NSString stringWithFormat:@"%s", policies[i].user];
    if ([user isEqualToString:kUserUri]) {
      NSString* title = [NSString stringWithFormat:@"%s", policies[i].name];
      NSString* value = [NSString stringWithFormat:@"%s", policies[i].uri];
      [policiesArray addObject:[NSDictionary dictionaryWithObjectsAndKeys:title, @"title", value, @"value", nil]];
    }
  }
  
  // check if the policy in userDefaults actually exists
  NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
  NSString* selectedPolicy = [[defaults objectForKey:@"policy"] valueForKey:@"value"];
  BOOL found = NO;
  for (NSDictionary* policy in policiesArray) {
    if ([selectedPolicy isEqualToString:[policy valueForKey:@"value"]]) {
      found = YES;
    }
  }
  if (!found) {
    [defaults setObject:nil forKey:@"policy"];
    [defaults synchronize];
  }
  
  // setup the policy controlller
  [MenuViewController setValue:policiesArray forKey:@"policies"];
  [MenuViewController setNavigationController:self.navigationController];
  
  self.policyController = [[MenuViewController alloc] initWithPlist:@"Policy"];
  self.policyController.delegate = self;
  
  // disable scrolling, since the content is larger than then space due to some padding on the bottom
  self.policyController.tableView.scrollEnabled = NO;

  CGRect frame = CGRectMake(0, 0, self.menuView.frame.size.width, self.menuView.frame.size.height);
  self.policyController.view.frame = frame;
  [self.menuView addSubview:self.policyController.view];
}

- (void)scrollViewDidScroll:(UIScrollView *)sender {
    // Update the page when more than 50% of the previous/next page is visible
    CGFloat pageWidth = self.scrollView.frame.size.width;
    int page = floor((self.scrollView.contentOffset.x - pageWidth / 2) / pageWidth) + 1;
  
    // flash the light, if page has changed
    if (page != self.pageControl.currentPage) {
      LightViewController* controller = [self.lightControllers objectAtIndex:page];
      KPFlashLight(controller.setting->lightUri);
    }
  
    self.pageControl.currentPage = page;
}

-(void) loadPolicy:(char*)uri {
  // clean up old policy
  for (LightViewController* controller in self.lightControllers) {
    [controller.view removeFromSuperview];
  }
  [self.lightControllers removeAllObjects];

  // get the settings for this policy
  KPLightPolicySetting settings[kKPMaxLightPolicySettings];
  int numSettings = KPGetLightPolicySettings(uri, settings, kKPMaxLightPolicySettings);
  
  for (int i = 0; i < numSettings; i++) {
    // get the name of the light of this setting (if any)
    NSString* name =  nil;
    if (settings[i].lightUri) {
      KPLight light;
      KPGetLight(settings[i].lightUri, &light);
      name = [NSString stringWithFormat:@"%s", light.name];
    } else {
      name = [NSString stringWithFormat:@"%s", KPLightTypeToString(settings[i].type)];
    }
    
    // add the light controller to the scroll view
    switch (settings[i].type) {
      case KPLightTypeDiscrete:
        [self addDiscreteLight:name setting:settings[i]];
        break;
      case KPLightTypeBrightness:
        [self addBrightnessLight:name setting:settings[i]];
        break;
      default:
        NSLog(@"Unknonw policy setting type %u", settings[i].type);
        break;
    }
  }
}

-(void) addLight:(LightViewController*)controller name:(NSString*)name setting:(KPLightPolicySetting)setting  {
  [self.lightControllers addObject:controller];
  
  // place the view at the end
  CGRect frame = controller.view.frame;
  frame.origin.x = ([self.lightControllers count] - 1)*kViewWidth;
  controller.view.frame = frame;
  
  // add the light controller to the view and enlarge the scrollview accordingly
  [self.scrollView addSubview:controller.view];
  self.scrollView.contentSize=CGSizeMake([self.lightControllers count]*kViewWidth, self.scrollView.frame.size.height);
  self.pageControl.numberOfPages = [self.lightControllers count];
  
  // number each light for ease of reference
  controller.nameLabel.text = name;
  strcpy(controller.setting->uri, setting.uri);
  strcpy(controller.setting->lightUri, setting.lightUri);
  controller.setting->type = setting.type;
}

-(void) addDiscreteLight:(NSString*)name setting:(KPLightPolicySetting)setting {
  DiscreteLightViewController* controller = [[DiscreteLightViewController alloc] initWithNibName:@"DiscreteLightView" bundle:[NSBundle mainBundle]];
  [self addLight:controller name:name setting:setting];
  KPGetDiscreteLightValue(setting.uri, controller.value);
  [controller update];
  [controller release];
}

-(void) addBrightnessLight:(NSString*)name setting:(KPLightPolicySetting)setting  {
  BrightnessLightViewController* controller = [[BrightnessLightViewController alloc] initWithNibName:@"BrightnessLightView" bundle:[NSBundle mainBundle]];
  [self addLight:controller name:name setting:setting];
  KPGetBrightnessLightValue(setting.uri, controller.value);
  [controller update];
  [controller release];
}

-(void) onPolicyChange:(id)policy {
  char* uri = (char*)[[policy valueForKey:@"value"] UTF8String];;
  [self loadPolicy:uri];
}

-(IBAction) onPolicySet {
  for (int i = 0; i < [self.lightControllers count]; i++) {
    LightViewController* controller = [self.lightControllers objectAtIndex:i];
    switch (controller.setting->type) {
      case KPLightTypeDiscrete:
        KPUpdateDiscreteLight(controller.setting->uri, ((DiscreteLightViewController*)controller).value);
        break;
      case KPLightTypeBrightness:
        KPUpdateBrightnessLight(controller.setting->uri, ((BrightnessLightViewController*)controller).value);
        break;
      default:
        NSLog(@"setPolicy: unknown controller class");
    }
  }
  
  // enact the selected policy
  NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
  NSString* selectedPolicy = [[defaults objectForKey:@"policy"] valueForKey:@"value"];
  if (!selectedPolicy) {
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"No policy selected" 
                                                    message:@"You must select the policy." 
                                                   delegate:nil 
                                          cancelButtonTitle:@"OK"
                                          otherButtonTitles:nil];
    [alert show];
    [alert release];
    return;
  }
  
  NSString* selectedUser = [defaults objectForKey:@"user"];
  if (!selectedUser) {
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"No user name set" 
                                                    message:@"You must specify the user name in the settings." 
                                                   delegate:nil 
                                          cancelButtonTitle:@"OK"
                                          otherButtonTitles:nil];
    [alert show];
    [alert release];
    return;
  }

  char* policy = (char*)[selectedPolicy UTF8String];
  char* user = (char*)[selectedUser UTF8String];
  KPEnactLightPolicy(user, policy);
}

#pragma mark Settings

-(void) onSettingsPress:(id)sender {
  MenuViewController* controller = [[MenuViewController alloc] initWithPlist:@"Settings"];
  controller.delegate = self;
  [self.navigationController pushViewController:controller animated:YES];
  [controller release];
}

-(void) onJoinPress {
  KPLeave();
  NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
  char* spaceId = (char*)[[defaults stringForKey:@"spaceId"] UTF8String];
  char* nodeId = (char*)[[defaults stringForKey:@"nodeId"] UTF8String];
  char* ip = (char*)[[defaults stringForKey:@"ip"] UTF8String];
  int port = [defaults integerForKey:@"port"];
  NSLog(@"Joiniong space=%s, node=%s, ip=%s, port=%d", spaceId, nodeId, ip, port);
  int joined = KPJoin(spaceId, nodeId, ip, port);
  [self viewRefresh];

  if (joined < 0) {
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Couldn't join the smart space"
                                                    message:@"Make sure the SIB settings are correct and that the SIB is running." 
                                                   delegate:nil 
                                          cancelButtonTitle:@"OK"
                                          otherButtonTitles:nil];
    [alert show];
    [alert release];
  } else {
    [self.navigationController popToRootViewControllerAnimated:YES];
  }
}

@end
